from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

def paragraph(text, width=None):
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager:

    recipe_name = "espda_coadd_spec"

    def setInteractiveParameters(self):
        paramList = list()
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "equal-flag", group = "Equalization", description =
            "Flag to equalize spectra"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "equal-ref",  group = "Equalization", description =
            "ID number of the reference exposure to be used in equalization: " \
            "[0, number of exposures]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "kappa",      group = "Equalization", description =
            "Threshold for kappa-sigma clipping to be used in rebinning: " \
            "[3, 5]"))
        #paramList.append(reflex.RecipeParameter(
        #    recipe=self.recipe_name,
        #    displayName="rebin-flag",
        #    group="Rebinning",
        #    description="Flag to rebin spectra: TRUE, FALSE (disregarded if "
        #        "only SPEC_1D is present)"
        #    ))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "wavel-min",  group = "Rebinning", description =
            "Minimum wavelength [nm]: [0.0, 1.0e5]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "wavel-max",  group = "Rebinning", description =
            "Minimum wavelength [nm]: [0.0, 1.0e5]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "vel-step",   group = "Rebinning", description =
            "Velocity step [km/s]: [0.3, 100.0]"))
        paramList.append(reflex.RecipeParameter(
            recipe=self.recipe_name,
            displayName="rv",
            group="Shifting",
            description="Radial velocity [km/s] (only for stellar spectra) [a "
                "list of comma-separated values for the different exposures, "
                "or a single value to be applied to all of them]"
            ))
        return paramList

    def readFitsData(self, fitsFiles):
        self.wavel_range = list()
        self.frames = dict()
        for frame in fitsFiles:
            if frame == '':
                continue
            category = frame.category
            self.frames[category] = frame

        self.rspec = PlotableData(self.frames["RSPEC_PRE"], e="FLUXERR")
        self.rsnr  = PlotableData(self.frames["RSPEC_PRE"], y = "SNR")
        self.rv_mean = 0
        self.rv_mean = self.rspec.fits.readKeyword("HIERARCH ESO DAS RV MEAN")
        #self.rspec_orig = PlotableData(self.frames["RSPEC_ORIG"],
        #    ref_fits = self.frames["RSPEC_PRE"], color = "green")
        #self.rsnr_orig  = PlotableData(self.frames["RSPEC_ORIG"],
        #    ref_fits = self.frames["RSPEC_PRE"], y = "SNR", color = "green")
        if "FSPEC_PRE"  in self.frames:
            self.fspec = PlotableData(self.frames["FSPEC_PRE"],
                ref_fits = self.frames["RSPEC_PRE"], color = "lightgrey")
            self.fsnr = PlotableData(self.frames["FSPEC_PRE"],
                ref_fits = self.frames["RSPEC_PRE"], y = "SNR",
                color = "lightgrey")

    def addSubplots(self, figure):
        gs_spec = gridspec.GridSpec(1, 1, left = 0.15, bottom = 0.45, top = 0.95)
        gs_snr  = gridspec.GridSpec(1, 1, left = 0.15, bottom = 0.05, top = 0.35)
        self.subplot_spec = figure.add_subplot(gs_spec[0,0])
        self.subplot_snr  = figure.add_subplot(gs_snr [0,0])

    def plotProductsGraphics(self):
        self.plotSpec()
        self.firstClick = False

    def plotSpec(self):
        spec_title   = "Spectrum (rest-frame; mean RV correction: %3.2f km/s)"\
                       % self.rv_mean
        spec_tooltip = "Grey: non-rebinned data; green: rebinned data; blue: "\
                       "equalized data.\n" \
                       "Click twice to change the wavelength range.\n" \
                       "Double-click to use the full wavelength range."
        spec_y_label = r"Flux density [erg $\mathdefault{cm^2}$ " \
                       r"$\mathdefault{s^{-1}}$ $\mathdefault{nm^{-1}}$]"
        snr_title    = "SNR per pixel"
        snr_tooltip  = "Blue: equalized data."
        snr_y_label  = r"SNR"
        all_x_label  = "Wavelength [nm]"

        #if "FSPEC_PRE"  in self.frames:
        #    self.fspec.plot(self.subplot_spec)
        #self.rspec_orig.plot(self.subplot_spec)
        self.rspec.plot(self.subplot_spec, title = spec_title,
            tooltip = spec_tooltip, x_label = all_x_label,
            y_label = spec_y_label, bars = self.wavel_range)

        #if "FSPEC_PRE"  in self.frames:
        #    self.fsnr.plot(self.subplot_snr)
        #self.rsnr_orig.plot(self.subplot_snr)
        self.rsnr.plot(self.subplot_snr, title = snr_title,
            tooltip = snr_tooltip, x_label = all_x_label, y_label = snr_y_label)

    def plotWidgets(self):
        widgets = list()
        self.clicks = 0
        self.clickable_spec = InteractiveClickableSubplot(self.subplot_spec,
            self.wavelRange)
        widgets.append(self.clickable_spec)
        return widgets

    def wavelRange(self, point):
        self.clicks = self.clicks + 1
        range_limit = self.rspec.getXPoint(point.xdata)
        if self.clicks < 3:
            if range_limit in self.wavel_range:
                self.wavel_range.remove(range_limit)
            else:
                self.wavel_range.append(range_limit)
        self.plotSpec()
        new_params = list()
        if self.clicks == 2:
            if len(self.wavel_range) == 2:
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-min",
                    value = str(self.wavel_range[0])))
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-max",
                    value = str(self.wavel_range[1])))
            else:
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-min",
                    value = str(380.0)))
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-max",
                    value = str(780.0)))
        return new_params

    def setWindowTitle(self):
      title = 'ESPRESSO Data Analysis Software: Spectrum Coaddition'
      return title

if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
